package com.grapeshot.halfnes;

import java.awt.EventQueue;
import javax.swing.SwingWorker;
import java.io.*;
//HalfNES, Copyright Andrew Hoffman, October 2010

public class utils {

    public static boolean getbit(final int num, final int bitnum) {
        //returns the nth bit of the int provided
        //bit numbers are zero indexed
        return ((num & (1 << bitnum)) != 0);
    }

    public static int getbitI(final int num, final int bitnum) {
        //returns the nth bit of the int provided
        //as an int, instead. Speeds up things when what you want is an int not a boolean to switch off.
        return (num >> bitnum) & 1;
    }

    public static boolean getbit(final long num, final int bitnum) {
        //returns the nth bit of the int provided
        //bit numbers are zero indexed
        return ((num & (1 << bitnum)) != 0);
    }

    public static String hex(final int num) {
        return Integer.toHexString(num).toUpperCase();
    }

    public static int reverseByte(int nibble) {
        //reverses 8 bits packed into int.
        return (Integer.reverse(nibble) >> 24) & 0xff;
    }

    public static void writetofile(final int[] array, final File file) {
        //note: does NOT write the ints directly to the file - only the low bytes.
        AsyncWriter writer = new AsyncWriter(array, file);
        writer.run();
    }

    public static void asyncwritetofile(final int[] array, final File file) {
        //now does the file writing in the dispatch thread
        //hopefully that will eliminate annoying hitches when file system's slow
        //and not do pathological stuff like threads are prone to
        AsyncWriter writer = new AsyncWriter(array, file);
        EventQueue.invokeLater(writer);
    }

    private static class AsyncWriter implements Runnable {

        private int[] a;
        private File f;

        public AsyncWriter(final int[] a, final File f) {
            this.a = a;
            this.f = f;
        }

        public void run() {
            if (a != null && f != null) {
                try {
                    FileOutputStream b = new FileOutputStream(f);
                    byte[] buf = new byte[a.length];
                    for (int i = 0; i < a.length; ++i) {
                        buf[i] = (byte) (a[i] & 0xff);
                    }
                    b.write(buf);
                    b.flush();
                    b.close();
                } catch (IOException e) {
                    System.err.print("Could not save. ");
                    System.err.println(e);
                }
            }
        }
    }

    public static int[] readfromfile(final File f) {
        byte[] bytes = new byte[(int) f.length()];
        FileInputStream fis;
        try {
            fis = new FileInputStream(f);

            try {
                fis.read(bytes);
                fis.close();
            } catch (IOException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        } catch (FileNotFoundException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        int[] ints = new int[bytes.length];
        for (int i = 0; i < bytes.length; i++) {
            ints[i] = (short) (bytes[i] & 0xFF);
        }
        return ints;
    }

    public static void printarray(final int[] a) {
        StringBuilder s = new StringBuilder();
        for (int i : a) {
            s.append(i);
            s.append(", ");
        }
        s.append("\n");
        System.err.print(s.toString());
    }

    public static void printarray(final short[] a) {
        StringBuilder s = new StringBuilder();
        for (short i : a) {
            s.append(i);
            s.append(", ");
        }
        s.append("\n");
        System.err.print(s.toString());
    }

    public static void printarray(final double[] a) {
        StringBuilder s = new StringBuilder();
        for (double i : a) {
            s.append(i);
            s.append(", ");
        }
        s.append("\n");
        System.err.print(s.toString());
    }

    public static String getExtension(final File f) {
        return getExtension(f.getName());


    }

    public static String getExtension(final String s) {
        if (s == null || s.equals("")) {
            return "";
        }
        int split = s.lastIndexOf(".");
        if (split < 0) {
            return "";
        }
        return s.substring(split);

    }

    public static String stripExtension(final File f) {
        String s = f.getName();
        if (s == null || s.equals("")) {
            return "";
        }
        int split = s.lastIndexOf(".");
        if (split < 0) {
            return "";
        }
        return s.substring(0, split);
    }

    public static int max(final int[] array) {
        int m = array[0];
        for (Integer i : array) {
            if (i > m) {
                i = m;
            }
        }
        return m;
    }
}
